import { defineComponent, nextTick, onMounted, ref, getCurrentInstance, isVue3, h, isVue2 } from 'vue-demi';
import { hh } from './DemiHelper';
import { InvokerContext } from '../types/InvokerContext';

export default defineComponent({
    name: 'Receiver',
    props: {
        parentId: {
            type: [Number, String],
            default: () => 0
        },
        parentKey: {
            type: String,
            default: () => 'MES_MOD_INVOKERS'
        }
    },
    setup(props) {
        const instance = getCurrentInstance();
        let invokerContext: InvokerContext = null!;
        const renderVersion = ref(0);
        console.log(`[Mod-Receiver] 2. receiver setup, invokerId: ${props.parentId}, invokerKey: ${props.parentKey}`);
        if (window.parent != window) {
            const invoker = window.parent[props.parentKey][props.parentId];
            invokerContext = invoker;
            console.log("[Mod-Receiver] 3. got invoker context", invokerContext);
        }
        else {
            throw new Error("[Mod-Receiver] the receiver must be in an iframe");
        }

        const renderItems = ref(invokerContext.getRenderContext() ?? []);

        // 若Invoker未接收到当前实例(首次初始化)，则进行初始化
        if (!invokerContext.receiver) {
            invokerContext.initFinish({
                get renderVersion() {
                    return renderVersion.value;
                },
                Update: function (): void {
                    renderVersion.value += 1;
                    renderItems.value = invokerContext.getRenderContext() ?? [];
                    instance?.proxy?.$forceUpdate();
                },
                // @ts-ignore
                get refs() {
                    return instance?.proxy?.$refs;
                }
            });
        }

        return (createElem) => {
            const render = isVue2 ? createElem : h;
            try {
                if (renderVersion.value < 0) return;
                if (invokerContext.getRenderContext) {
                    const itemList = invokerContext.getRenderContext() ?? [];

                    // const list = renderContext(itemList);
                    console.log("[Mod-Receiver] 4. rendering: ", itemList);
                    // @ts-ignore
                    return hh('div', undefined, itemList, render);
                }
                console.warn("[Mod-Receiver] 'getRenderContext' method not found");
                return hh('div', undefined, undefined, render);
            }
            catch (e) {
                console.error("[Mod-Receiver] 渲染节点过程中出现错误", e);
            }
        };
    }
});
