import { ComponentOptions, nextTick, VNode, WatchOptions } from "vue-demi"

export interface Vue2Instance<
    Data = Record<string, any>,
    Props = Record<string, any>,
    Instance = never,
    Options = never,
    Emit = (event: string, ...args: any[]) => Vue2Instance
> {
    // properties with different types in defineComponent()
    readonly $data: Data
    readonly $props: Props
    readonly $parent: NeverFallback<Instance, Vue2Instance> | null
    readonly $root: NeverFallback<Instance, Vue2Instance>
    readonly $children: NeverFallback<Instance, Vue2Instance>[]
    readonly $options: NeverFallback<Options, ComponentOptions<Vue2Instance>>
    $emit: Emit

    // Vue 2 only or shared
    readonly $el: Element
    readonly $refs: {
        [key: string]:
        | NeverFallback<Instance, Vue2Instance>
        | Vue2Instance
        | Element
        | (NeverFallback<Instance, Vue2Instance> | Vue2Instance | Element)[]
        | undefined
    }
    readonly $slots: { [key: string]: VNode[] | undefined }
    readonly $scopedSlots: { [key: string]: NormalizedScopedSlot | undefined }
    readonly $isServer: boolean

    readonly $ssrContext: any
    readonly $vnode: VNode
    readonly $attrs: Record<string, string>
    readonly $listeners: Record<string, Function | Function[]>

    $mount(elementOrSelector?: Element | string, hydrating?: boolean): this
    $forceUpdate(): void
    $destroy(): void
    $set: Function
    $delete: Function
    $watch(
        expOrFn: string,
        callback: (this: this, n: any, o: any) => void,
        options?: WatchOptions
    ): () => void
    $watch<T>(
        expOrFn: (this: this) => T,
        callback: (this: this, n: T, o: T) => void,
        options?: WatchOptions
    ): () => void
    $on(event: string | string[], callback: Function): this
    $once(event: string | string[], callback: Function): this
    $off(event?: string | string[], callback?: Function): this
    $nextTick: typeof nextTick
    $createElement: Function
}

type NeverFallback<T, D> = [T] extends [never] ? D : T

export type NormalizedScopedSlot = (props: any) => ScopedSlotChildren
export type ScopedSlotChildren = VNode[] | undefined
